import json
from typing import Union

from adara_privacy.identity import Identity
from adara_privacy.token import Token
from adara_privacy.streamers.streamer import Streamer
from adara_privacy.utils.strings import is_empty_string


class FileStreamer(Streamer):
    """
    Saves and reads identity tokens to/from disk in a consistent file format.
    """

    def __init__(self, file_name: str, file_format: str = 'token'):
        """
        Creates a new instance of a FileStreamer.

        Args:
            file_name (str): Path and filename (with extension) of token set data file
            file_format (str): Defines whether the file is for data of type "token" or "identity" (default: "token")
        """
        super().__init__()

        self._mode = 'r'

        # validate arg: file_name
        if isinstance(file_name, str) and not is_empty_string(file_name):
            self._filename = file_name
        else:
            raise ValueError('Argument "file_name" must be a non-empty string.')

        # validate arg: file_format
        if isinstance(file_format, str) and file_format.strip().lower() in {'token', 'identity'}:
            self._file_format = file_format.strip().lower()
        else:
            raise ValueError('Argument "file_format" must be either "token" or "identity".')

        # init instance defaults
        self._file = None  # stores the file handle after opening

    def __enter__(self):
        """
        Supports the "with" syntax
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        Supports the "with" syntax
        """
        self.close()
        return

    @property
    def _is_file_open(self) -> bool:
        """
        Private method to return whether a file handle is currently open.

        Returns:
            bool: True if there is an open handle, otherwise False.
        """
        return self._file is not None and not self._file.closed

    def open(self, mode: str = 'read'):
        """
        Opens the file represented by the file_name argument passed to the constructor.

        Args:
            mode (str): Mode for opening the file.  Options are:
                "read" / "r" : opens file for reading only
                "append" / "a" : opens file for appended writes (will not overwrite existing, appends data starting at end of file)
                "write" / "w" : opens file for writing; WARNING: will overwrite any existing file with the same name

                NOTE: Both "append" and "write" modes will create the file if it does not already exist
        """

        if isinstance(mode, str):
            mode = mode.strip().lower()
            if mode in {'read', 'r'}:
                self._mode = 'r'
            elif mode in {'append', 'a'}:
                self._mode = 'a'
            elif mode in {'write', 'w'}:
                self._mode = 'w'
            else:
                raise ValueError('Argument "mode" must be one of "r", "a", or "w".')
        else:
            raise TypeError('Argument "mode" must be one of "r", "a", or "w".')

        # close any currently open handles
        if self._is_file_open:
            self._file.close()

        self._file = open(file=self._filename, mode=self._mode)

    def close(self):
        """
        Closes the file.
        """
        if self._is_file_open:
            self._file.close()

    def save(self, item: Union[Token, Identity]):
        """
        Saves tokens to the file. Automatically opens the file if it's not already open.

        Args:
            identity (Identity): An instance of an Identity that contains tokens to write to the file.
        """
        # arg check: item type
        if not (isinstance(item, (Token, Identity)) or (isinstance(item, list) and all(isinstance(t, Token) for t in item))):
            raise TypeError('Argument "item" must be an instance of either Token or Identity (list of Token instances also accepted).')

        # arg check: item type matches self._file_format
        if isinstance(item, Token) and self._file_format == 'identity':
            # note that if format is token and we're passed an identity, that's fine as we can get the token
            raise TypeError('Argument "item" was passed an instance of Token but file streamer is formatted for Identity.')

        # ensure the file is open
        if not self._is_file_open:
            self.open('append')

        # choose the object type to write
        if isinstance(item, Token):
            data = item.to_dict()
        elif isinstance(item, Identity):
            if self._file_format == 'token':
                data = item.to_dict(format='token')
            else:
                data = item.to_dict()
        else:  # based on check above, we know this is a list of tokens
            data = [t.to_dict() for t in item]

        # write the record in a single line
        # prefix the string with newline if there are previous records
        self._file.write(
            json.dumps(data) + '\n'
        )
        self._records_written = True

    def read(self) -> Union[Token, Identity]:
        """
        Reads tokens from the file. Automatically opens the file if it's not already open.
        This is a generator that supports iteration for line-by-line read operations.

        Yields:
            Identity: Returns an instance of an identity containing the tokens read from a single line of the file.
        """
        # ensure the file is open
        if not self._is_file_open:
            self.open('read')

        # read line by line and return instances based on self._file_format
        for line in self._file:
            if self._file_format == 'token':
                yield Token(json.loads(line))
            else:
                yield Identity(json.loads(line))



import requests
import logging
import json

from adara_privacy.utils import sdk_config
from adara_privacy.identity import Identity
from adara_privacy.streamers.streamer import Streamer


class AdaraPrivacyApiStreamer(Streamer):
    """
    Streams data to the Adara Privacy API.
    ** Requires a valid configuration to establish an authenticated connection to a provisioned client pipeline.
    """

    def __init__(self):
        super().__init__()

    def save(self, item: Identity, data=None):
        """
        Pushes an identity's token set to the Adara Privacy API.

        Args:
            item (Identity): An instance of an Identity that contains tokens to send
            to the Adara Privacy API.
            data: Optional arbitrary data to augment identity information
        """
        if isinstance(item, Identity):
            try:
                tokens = item.to_tokens()
                if data:
                    if isinstance(data, (str, bytes, bytearray)):
                        try:
                            data = json.loads(data)
                            tokens['data'] = data
                        except json.decoder.JSONDecodeError as errj:
                            logging.warning(
                                "Supplied arbitrary data cannot be decoded to JSON!"
                                " Data will not be added to the request. Error: {}".format(
                                    errj))
                    else:
                        tokens['data'] = data
                response = requests.post(
                    url=f'{sdk_config.privacy.audience_uri}/v2/send/{sdk_config.privacy.pipeline_id}',
                    headers={
                        'Content-Type': 'application/json',
                        'Authorization': "Bearer " + sdk_config.privacy.privacy_api_access_token,
                    },
                    data=json.dumps(tokens)
                )
                response.raise_for_status()
            except requests.exceptions.HTTPError as errh:
                logging.error("Http Error:", errh)
            except requests.exceptions.ConnectionError as errc:
                logging.error("Error Connecting:", errc)
            except requests.exceptions.Timeout as errt:
                logging.error("Timeout Error:", errt)
            except requests.exceptions.RequestException as err:
                logging.error("Other Request Error:", err)
        else:
            raise TypeError('Argument "item" must contain a reference to an instance of Identity.')



class Streamer():
    """
    Abstract base class to support streaming operations for token transport.
    """

    def __init__(self):
        super().__init__()

    def __enter__(self):
        """
        Supports the "with" syntax
        """
        return self

    def __exit__(self, type, value, traceback):
        """
        Supports the "with" syntax
        """
        return

    def save(self, identity):
        """
        Abstract method for saving tokens to stream.
        """
        raise NotImplementedError('This method cannot be used on the abstract base class.')



class InvalidConfigurationFormatError(Exception):
    def __init__(self, *args, **kwargs):
        super().__init__('The file referenced by the ADARA_SDK_CREDENTIALS environment variable is not in a valid JSON format and cannot be read.', *args, **kwargs)



EMPTY_VALUES = {
    '',
    'null',
    'none',
    '0',
    '-',
    'na',
    'n/a',
}


def is_stringable(value) -> str:
    return isinstance(value, (str, int, float))


def is_empty_string(s: str) -> str:
    return not s or s.strip().lower() in EMPTY_VALUES


def identifiers_as_single_str(*args: str) -> str:
    """
    Private function for standardizing string coalescence for multi-value tokenization routines.

    Returns:
        str: Returns the single string output
    """
    return ':'.join(sorted(arg.strip().lower() for arg in args))




from adara_privacy.utils import (
    is_stringable,
    is_empty_string,
    sdk_config,
)
import adara_privacy.token as tokens

IDENTIFIER_KEYWORDS = {
    'social_security': {'social_security'},
    'passport': {'passport'},
    'drivers_license': {'drivers_license'},
    'state_id': {'state_id'},
    'email': {'email'},
    'hashed_email': {'hashed_email'},  # TODO: Change to sha256 name
    'membership_id': {'membership_id'},
    'customer_id': {'customer_id'},
    'cookie': {'cookie'},
    'streetname_zipcode': {'street_name', 'zip_code'},
}


class Identifier():
    KEY_GENERIC_IDENTIFIER = "general"

    def __init__(self, *args, identifier_type: str = None, identifier_dict: dict = None, common_tokens: list = None, label: str = None, **kwargs):
        super().__init__()
        self._general_values = []
        self._named_values = dict()
        self._identifier_type = None
        self._token = None
        self._common_tokens = None
        self._label = None

        if identifier_type:
            if isinstance(identifier_type, str) and identifier_type.strip().lower() in {k for k in IDENTIFIER_KEYWORDS.keys()}:
                self._identifier_type = identifier_type.strip().lower()
            else:
                raise TypeError('Argument "identifier_type" be must a valid string value representing a known identifier type.')

        if common_tokens:
            if isinstance(common_tokens, list) and all(isinstance(i, str) for i in common_tokens):
                self._common_tokens = list(common_tokens)
            elif isinstance(common_tokens, str):
                self._common_tokens = [t.strip() for t in common_tokens.replace(';', ',').split(',') if t.strip() != '']
            else:
                raise TypeError('Argument "common_tokens" must be a string (CSV accepted) or list of strings.')

        if label:
            if isinstance(label, str):
                self._label = label.strip()
            elif label:
                raise TypeError('Argument "label" must be of type "str".')
            else:
                self._label = None

        if identifier_dict:
            if isinstance(identifier_dict, dict):
                self._identifier_type, self._general_values, self._named_values = self._extract_values(self._identifier_type, **identifier_dict)
            else:
                raise TypeError('Argument "identifier_dict" must be of type "dict".')
            if identifier_type or len(args) > 0 or len(kwargs) > 0:
                raise ValueError('When argument "identifier_dict" is provided, it must be the only argument.')

        elif args or kwargs:

            if len(args) == 1 and isinstance(args[0], dict) and not kwargs:
                self._identifier_type, self._general_values, self._named_values = self._extract_values(self._identifier_type, **args[0])
            elif (not args or all(is_stringable(arg) for arg in args)) and (not kwargs or all(is_stringable(kwargs[k]) for k in kwargs.keys())):
                self._identifier_type, self._general_values, self._named_values = self._extract_values(self._identifier_type, *args, **kwargs)
                if not sdk_config.privacy.ignore_empty_identifiers and not self._general_values and not self._named_values:
                    raise ValueError('At least one identifier argument must be passed as a non-empty string or number value.  You can skip this error by setting the configuration "privacy.ignore_empty_identifiers" to True.')
            else:
                raise TypeError('Positional arguments must be either all strings (used as identifiers) or must be a single instance of "dict".')
        else:
            raise TypeError('Missing argument value: at least one identifier argument must be passed as a non-empty string or number value.')

    def _extract_values(self, current_identifier_type: str, *args, **kwargs) -> (str, list, dict):
        general_values = list()
        derived_identifier_type = current_identifier_type

        for arg in args:
            # this allows a positional argument to specify the identifier type
            if not derived_identifier_type and arg.strip().lower() in {k for k in IDENTIFIER_KEYWORDS.keys()}:
                derived_identifier_type = arg.strip().lower()
            elif not is_empty_string(arg):
                general_values.append(arg)

        # check kwargs for a identifier combo that we can match
        if not derived_identifier_type:
            id_keys = {k.lower() for k in kwargs}
            for k in IDENTIFIER_KEYWORDS:
                if IDENTIFIER_KEYWORDS[k] == id_keys:
                    derived_identifier_type = k
                    break

            if not derived_identifier_type:
                id_types = set(IDENTIFIER_KEYWORDS.keys())
                for k in id_keys:
                    if isinstance(kwargs[k], list) and k in id_types:
                        derived_identifier_type = k
                        break

        return derived_identifier_type, general_values, dict(**kwargs)

    @property
    def tokenization_value(self) -> str:
        # start with the positional argument values
        v = list(self._general_values)
        # add in keyword argument values
        for k in self._named_values:
            if isinstance(self._named_values[k], list):
                v.extend(self._named_values[k])
            else:
                v.append(self._named_values[k])
        # return the value list, sorted and lower-cased
        return ':'.join(sorted(v.strip().lower() for v in v))

    @property
    def token(self) -> tokens.Token:
        if not self._token and not self.is_empty:
            self._token = tokens.Token(self)
        return self._token

    @property
    def identifier_type(self) -> str:
        return self._identifier_type if self._identifier_type else Identifier.KEY_GENERIC_IDENTIFIER

    @property
    def label(self) -> str:
        return self._label

    @property
    def common_tokens(self) -> list:
        return self._common_tokens

    @property
    def is_empty(self) -> bool:
        return not (self._general_values or self._named_values)

    @property
    def values(self):
        general_values = list(self._general_values)
        named_values = dict()
        for k in self._named_values:
            if k == self._identifier_type:
                general_values.append(self._named_values[k])
            else:
                named_values[k] = self._named_values[k]
        general_values_dict = dict()
        if general_values:
            general_values_dict[self._identifier_type if self._identifier_type else 'value'] = general_values[0] if len(general_values) == 1 else general_values
        return {
            **general_values_dict,
            **named_values,
        }

    def to_dict(self):
        # TODO: add additional attributes here (ex: label)
        return self.values

    def __eq__(self, other):
        return True



import hashlib

from adara_privacy.utils import (
    sdk_config,
)

import adara_privacy

KEY_TOKEN_TYPE = 'type'
KEY_TOKEN_COMMON = 'common'
KEY_TOKEN_PRIVATE = 'private'


def _tokenize(value: str, salt: str) -> str:
    return hashlib.sha256(str(salt + hashlib.sha256(value.encode()).hexdigest()).encode()).hexdigest()


class Token():
    def __init__(self, *args, identifier=None, token_dict: dict = None):
        super().__init__()
        self._identifier = None
        self._common_tokens = None
        self._private = None

        if len(args) == 1:
            if identifier or token_dict:
                raise ValueError("Token constructor accepts a single input and multiple were given.")
            if isinstance(args[0], adara_privacy.Identifier):
                identifier = args[0]
            elif isinstance(args[0], dict):
                token_dict = args[0]
            else:
                raise TypeError("Token constructor only accepts either an Identifier instance or dict.")
        elif len(args) > 1:
            raise ValueError("Token constructor accepts a single input and multiple were given.")

        if identifier and token_dict:
            raise ValueError("Token constructor accepts a single input and multiple were given.")
        elif not (identifier or token_dict):
            raise ValueError("Token constructor requires a single input but none was given.")

        if isinstance(identifier, adara_privacy.Identifier):
            self._identifier = identifier
        elif isinstance(token_dict, dict):
            self._common_tokens = dict()
            # try:
            #     self._common = token_dict[KEY_TOKEN_COMMON]
            # except KeyError:
            #     raise ValueError('Argument "token_dict" was passed an invalid token definition: key "common" is required.')
            try:
                self._identifier_type = token_dict[KEY_TOKEN_TYPE]
            except KeyError:
                self._identifier_type = adara_privacy.Identifier.KEY_GENERIC_IDENTIFIER
            # try:
            #     assert self.private == token_dict[KEY_TOKEN_PRIVATE]
            # except KeyError:
            #     pass
            # except AssertionError:
            #     raise ValueError('Value for key "private" does not match expected salted token for "common" value.')
        else:
            raise Exception('Token() was not passed arguments required for instantiation.')

    @property
    def identifier_type(self):
        return self._identifier.identifier_type if self._identifier else self._identifier_type

    @property
    def common(self):
        if self._common_tokens is None:
            self._common_tokens = {k: _tokenize(self._identifier.tokenization_value, sdk_config.privacy.common_salts[k]) for k in sdk_config.privacy.common_salts}
        return self._common_tokens

    @property
    def private(self):
        if not self._private:
            self._private = _tokenize(self._identifier.tokenization_value, sdk_config.privacy.private_salt)
        return self._private

    def to_dict(self):
        tokens = {KEY_TOKEN_PRIVATE: self.private, **self.common}
        if self._identifier.common_tokens:
            tokens = {k: tokens[k] for k in tokens if k in set(self._identifier.common_tokens)}

        metadata = {KEY_TOKEN_TYPE: self.identifier_type}
        if self._identifier.label:
            metadata['label'] = self._identifier.label

        return {**tokens, **metadata}



from adara_privacy.identifier import Identifier
from adara_privacy.streamers.streamer import Streamer
from adara_privacy.token import _tokenize
from adara_privacy.utils import sdk_config


class Identity():
    def __init__(self, *args, **kwargs):
        super().__init__()
        self._identifiers = []

        if args and all(isinstance(arg, Identifier) for arg in args):
            for arg in args:
                if not arg.is_empty:
                    self._identifiers.append(arg)
        elif len(args) == 1 and isinstance(args[0], list) and all(isinstance(item, dict) for item in args[0]):
            self._identifiers.extend(Identifier(item) for item in args[0])
        elif len(args) > 0:
            raise TypeError('Arguments passed to constructor must be all of type Identifier, or a single argument as a list of deserializable token dicts.')

        if kwargs and all(isinstance(kwargs[k], (str, int, float)) for k in kwargs):
            self._identifiers.extend([Identifier(kwargs[k], identifier_type=k)] for k in kwargs)

    def add_identifier(self, identifier):
        if isinstance(identifier, Identifier):
            if not identifier.is_empty:
                self._identifiers.append(identifier)
        else:
            raise TypeError('Argument "identifier" must be an instance of type Identifier.')

    @property
    def identifiers(self):
        return list(self._identifiers)

    @property
    def tokens(self):
        return [
            identifier.token for identifier in self._identifiers
        ]

    @property
    def package_token(self):
        return _tokenize(':'.join(sorted({t.private + t.identifier_type for t in self.tokens})), sdk_config.privacy.private_salt)

    def save(self, streamer):
        if not isinstance(streamer, Streamer):
            raise TypeError('Argument "streamer" must be an instance of type "Streamer".')
        streamer.save(self)

    def to_dict(self, format: str = 'identity'):
        if format == 'identity':
            return {
                'package_token': self.package_token,
                'identities': [i.to_dict() for i in self._identifiers]
            }
        elif format == 'token':
            return self.to_tokens()
        else:
            raise ValueError('Argument "format" must be either "identity" (default) or "token".')

    def to_tokens(self):
        return {
            'package_token': self.package_token,
            'tokens': [i.token.to_dict() for i in self._identifiers]
        }

    def __eq__(self, other):
        return True



class MissingRequiredSettingError(Exception):
    def __init__(self, setting_name: str, *args, **kwargs):
        super().__init__(f'Missing required setting "{setting_name}".  Please check the configuration file to ensure this value has been added.', *args, **kwargs)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="adara_privacy",
    version='0.1.2',
    author="Adara, Inc.",
    author_email="oss@adara.com",
    description="The Adara Privacy SDK is an open source library which allows you to safely manage sensitive Personally Identifiable Information (PII).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/adarainc/adara-privacy-sdk-python",
    packages=['adara_privacy'],
    include_package_data=True,
    scripts=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
        "requests"
    ]
)



import os
import json
import requests
import datetime

from adara_privacy.exceptions import (
    MissingEnvironmentVariableError,
    MissingRequiredSettingError,
    InvalidConfigurationFormatError,
)

CONFIG_ENVIRONMENT_VARIABLE = 'ADARA_SDK_CREDENTIALS'

# top-level config keys
CONFIG_KEY_CLIENT_ID = 'client_id'
CONFIG_KEY_CLIENT_SECRET = 'client_secret'
CONFIG_KEY_AUTH_URI = 'auth_uri'

# privacy config keys
CONFIG_KEY_PRIVACY = 'privacy'  # section key

CONFIG_KEY_PRIVATE_SALT = 'private_salt'
CONFIG_KEY_COMMON_SALT = 'common_salt'
CONFIG_KEY_COMMON_SALTS = 'common_salts'
CONFIG_KEY_AUDIENCE_URI = 'audience_uri'
CONFIG_KEY_PIPELINE_ID = 'pipeline_id'
CONFIG_KEY_IGNORE_EMPTY_IDENTIFIERS = 'ignore_empty_identifiers'

# reserved value names
RESERVED_PRIVATE = 'private'
RESERVED_COMMON = 'common'
RESERVED_TYPE = 'type'
RESERVED_LABEL = 'label'


def _get_config_keys(keys: list, config: dict) -> dict:
    result = dict()
    sought_keys = {k.strip().lower() for k in keys}
    for k in config.keys():
        key = k.strip().lower()
        if key in sought_keys:
            result[key] = config[k]

    return result


class PrivacyConfig():
    def __init__(self, parent_config, config_data: dict, *args, **kwargs):
        super().__init__()

        if not isinstance(config_data, dict):
            raise TypeError('Privacy configuration module must be a dictionary.')

        config = _get_config_keys([
            CONFIG_KEY_PRIVATE_SALT,
            CONFIG_KEY_COMMON_SALT,
            CONFIG_KEY_COMMON_SALTS,
            CONFIG_KEY_AUDIENCE_URI,
            CONFIG_KEY_PIPELINE_ID,
        ], config_data)

        # REQUIRED: Private Salt setting
        try:
            self._private_salt = config[CONFIG_KEY_PRIVATE_SALT]
        except KeyError:
            raise MissingRequiredSettingError(CONFIG_KEY_PRIVATE_SALT) from None

        self._common_salts = dict()
        # OPTIONAL: Common Salt setting
        # NOTE: This provides backward functionality to a single "common_salt" key; this is
        # deprecated with the introduction of an arbitrary number of named salts
        # TODO: REMOVE THIS!
        try:
            self._common_salts[RESERVED_COMMON] = config[CONFIG_KEY_COMMON_SALT]
        except KeyError:
            pass

        # OPTIONAL: Named Salts
        try:
            salts = config[CONFIG_KEY_COMMON_SALTS]
            assert isinstance(salts, dict)
            assert all(isinstance(salts[k], str) and k.strip().lower() != RESERVED_PRIVATE for k in salts)
            self._common_salts.update(salts)
        except KeyError:
            pass
        except AssertionError:
            raise TypeError(f'Configuration key "{CONFIG_KEY_COMMON_SALTS}" must contain a dictionary of key/value string pairs.')

        # OPTIONAL: Audience URI setting
        try:
            self._audience_uri = config[CONFIG_KEY_AUDIENCE_URI]
        except KeyError:
            self._audience_uri = None

        # OPTIONAL: Pipeline ID setting
        try:
            self._pipeline_id = config[CONFIG_KEY_PIPELINE_ID]
        except KeyError:
            self._pipeline_id = None

        # OPTIONAL: Ignore empty identifiers setting
        try:
            self._ignore_empty_identifiers = config[CONFIG_KEY_IGNORE_EMPTY_IDENTIFIERS]
        except KeyError:
            self._ignore_empty_identifiers = True

        self._parent_config = parent_config
        self._access_token = None
        self._access_token_expiry = None

    @property
    def private_salt(self):
        return self._private_salt

    @property
    def common_salts(self):
        return self._common_salts

    @property
    def audience_uri(self):
        return self._audience_uri

    @property
    def pipeline_id(self):
        return self._pipeline_id

    @property
    def ignore_empty_identifiers(self):
        return self._ignore_empty_identifiers

    def _get_access_token(self):
        r = requests.post(
            self._parent_config.auth_uri,
            json=dict(
                client_id=self._parent_config.client_id,
                client_secret=self._parent_config.client_secret,
                audience=self._parent_config.privacy.audience_uri,
                grant_type="client_credentials"
            )
        )
        r_json = r.json()
        self._access_token = r_json['access_token']
        self._access_token_expiry = datetime.datetime.utcnow() + datetime.timedelta(seconds=r_json['expires_in'] - 300)

    @property
    def privacy_api_access_token(self):
        if (not self._access_token) or (not self._access_token_expiry) or (
                isinstance(self._access_token_expiry, datetime.datetime) and datetime.datetime.utcnow() >= self._access_token_expiry):
            self._get_access_token()
        return self._access_token


class Config():
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        try:
            self._path = os.environ[CONFIG_ENVIRONMENT_VARIABLE]
        except KeyError as e:
            raise MissingEnvironmentVariableError() from None

        # read in the local file - must be valid json
        try:
            with open(self._path) as f:
                config_data = json.load(f)
        except FileNotFoundError as e:
            raise e from None
        except json.decoder.JSONDecodeError as e:
            raise InvalidConfigurationFormatError() from None
        except Exception as e:
            raise e from None

        # now parse out the settings
        config = _get_config_keys([
            CONFIG_KEY_CLIENT_ID,
            CONFIG_KEY_CLIENT_SECRET,
            CONFIG_KEY_AUTH_URI,
            CONFIG_KEY_PRIVACY,
        ], config_data)

        # OPTIONAL: Client ID
        try:
            self._client_id = config[CONFIG_KEY_CLIENT_ID]
        except KeyError:
            self._client_id = None

        # OPTIONAL: Client Secret
        try:
            self._client_secret = config[CONFIG_KEY_CLIENT_SECRET]
        except KeyError:
            self._client_secret = None

        # OPTIONAL: Authorization URI
        try:
            self._auth_uri = config[CONFIG_KEY_AUTH_URI]
        except KeyError:
            self._auth_uri = None

        # OPTIONAL: Privacy (config section)
        try:
            self._privacy = PrivacyConfig(self, config[CONFIG_KEY_PRIVACY])
        except KeyError:
            self._privacy = None

    @property
    def client_id(self):
        return self._client_id

    @property
    def client_secret(self):
        return self._client_secret

    @property
    def auth_uri(self):
        return self._auth_uri

    @property
    def privacy(self):
        return self._privacy



from adara_privacy.exceptions.missing_environment_variable_error import MissingEnvironmentVariableError
from adara_privacy.exceptions.missing_required_setting_error import MissingRequiredSettingError
from adara_privacy.exceptions.invalid_configuration_format_error import InvalidConfigurationFormatError
